function b = dec_tetro_rel_edge(a,J,num_cov,rel)
%
% filterbank decomposition algorithm for modified tetrolet transform
% 'tetrolet 16 rel edge'
%
% INPUT:
% a -        input image
% J -        number of decomposition level
% num_cov -  number of admissible coverings
%
% OUTPUT:
% b: cell array of size 2xJ with tetrolet coefficients
%
%
% CHANGELOG
%
%  2009-09-30 (jens.krommweh@uni-due.de)



% ### initialisation ###
global COV_DISTRIB
[WW,P] = init_tetro(num_cov);

[N1,N2] = size(a);

coef_cell = cell(J+1,2);  % first entry contains lowpass part, further entries the wave coefficients
distr = zeros(J,num_cov); % distribution of choosen covering
dis = zeros(1,num_cov);   % distribution (only needed for relaxed version)
coef_cell{1,1} = a;       % initialization with input image

% #####################
% ### Decomposition ###
% #####################

for zz = 1:J
    N1 = N1/2; N2 = N2/2;
    coef_high = cell(N1/2,N2/2);
    lowpass = zeros(N1);
    
    covering = edge_det(coef_cell{1,1},35); % edge detection ??? how to choose threshold ???
    %figure, imagesc(covering), axis square;
        
for ii = 1:N1/2
    for jj = 1:N2/2

        % blocking
        B = coef_cell{1,1}(4*ii-3:4*ii,4*jj-3:4*jj);

        pixel_matrix = zeros(size(P));
        pixel_matrix(:,1) = B(:);
        pixel_matrix = cumsum(pixel_matrix,2);
        
        % r: lowpass part, wave_min: wavelets, permut: number of covering
        if covering(ii,jj) == 0 % CASE egdes
            [r, wave_min, permut] = opt_wave10rel(pixel_matrix(:),dis,rel);
        else % CASE no edges
            haar = WW(1:16,1:16)*B(:); % Haar WT
            r = haar([1 5 9 13]);      % extract scf coeff (4er vector)
            haar([1 5 9 13],:) = [];   % delete scf coeff (12er vector)
            wave_min = haar;
            permut = 1;
        end

        % store wave coefficients with corresponding permutation vector
        coef_high{ii,jj} = wave_min; % 12er vector
        covering(ii,jj) = permut;    % scalar
        % distribution
        dis(permut) = dis(permut) + 1;
        % store lowpass part
        lowpass(2*ii-1:2*ii,2*jj-1:2*jj) = reshape(r,2,2);
    end
end
    coef_cell{1,1} = lowpass;  % lowpass-part
    coef_cell{zz+1,1} = coef_high;
    coef_cell{zz+1,2} = covering;
    %covering, %figure, hist(covering, 1:num_cov); % display covering distribution
    distr(zz,:) = sum(histc(covering, 1:num_cov,1),2);
    %figure, imagesc(lowpass/255), axis square;
end

COV_DISTRIB = sum(distr); % covering distribution ( = dis)
%figure, bar(COV_DISTRIB); % display covering distribution

b = coef_cell;