function b = rec_tetro(coef_cell)
%
% filter bank reconstruction algorithm
%
% INPUT
% coef_cell - cell vector of length J+1, which entries are cell arrays,
%            which entries are coefficients of the image blocks 
%
% OUTPUT
% b - reconstructed image
%
% 2009-02-20 (jens.krommweh@uni-due.de)

global P % contains permutations
global W % 16x16 Haar-Wavelet-Matrix
 
I = eye(16);

% ### reconstruction filter bank ###
coef_low = coef_cell{1,1};
[N1,N2] = size(coef_low);
coef_low_neu = zeros(2*[N1,N2]);

J = size(coef_cell,1)-1;

for zz = J:-1:1
    coef_high = coef_cell{zz+1,1};
    zuordnung = coef_cell{zz+1,2};
    for ii = 1:2:N1
        for jj = 1:2:N2
            r = coef_low(ii:ii+1,jj:jj+1);       % lowpass part
            wave = coef_high{(ii+1)/2,(jj+1)/2}; % highpass part, 12er vector
            r = r(:);
            wave = vertcat(r(1), wave(1:3), r(2), wave(4:6), r(3), wave(7:9), r(4), wave(10:12)); % add 4 scf coeff
            pival = W*wave;
            
            % permute back
            permut = zuordnung((ii+1)/2,(jj+1)/2);
            perm1 = P(:,permut);

            Phi = inv(I(perm1,:))*pival;
            coef_low_neu(2*ii-1:2*ii+2,2*jj-1:2*jj+2) = reshape(Phi,4,4);

        end
    end
    N1 = 2*N1; N2 = 2*N2;
    coef_low = coef_low_neu;
end
b = coef_low;
b(b < 0) = 0; b(b > 255) = 255;

% smoothing (post processing step)
m1 = min(b(:)); m2 = max(b(:));
b = (b-m1)/(m2-m1);               % normalization to [0,1]
b = bfilter2gray(b,1,[1 35/255]); % 2D bilateral filtering
b = b*(m2-m1)+m1;                 % re-normalization



