function coef_thresh = thresh_cell(coef_cell,N,request)

% global thresholding in cell array for COMPLETE decomposition
%
% INPUT
% coef_cell - cell vector of length j, which entries are cell arrays,
%             which entries are coefficients of the image blocks 
%        N  - number of coefficients after thresholding (resp. threshold parameter)
%   request - optional string: 'thresh', then N is the treshold parameter (not the number of coefficient!)
%
% OUTPUT
% coef_thresh - thresholded cell vector with same structure as coef_cell
%
%
% 2009-03-16 (jens.krommweh@uni-due.de)


% ### I) convert coef_cell into vector y ###

y = [];
for zz = 2:size(coef_cell,1) % start with zz = 2 -> without regarding scf coeff
     r = cell2mat(coef_cell{zz,1});
     y = [y; r(:)]; % column vector
end

%length(y)

% ### II) wavelet shrinkage ###

if nargin == 3
    % case 1:  threshold is given
    y_thresh = (abs(y) > N) .* y; % HARD
 %   y_thresh = SoftThresh(y,N);   % SOFT
else % case 2: desired number of coefficients is given
    y_thresh = zeros(size(y));
    [y1,permut] = sort(abs(y));
    permut(1:end-N) = [];
    y_thresh(permut) = y(permut);
end

% ### III) re-convert vector y_thresh into cell array coef_thresh

for zz = 2:size(coef_cell,1) % start with zz = 2 -> without regarding scf coeff
    [s1,s2] = size(coef_cell{zz,1});
    [t1,t2] = size(coef_cell{zz,1}{1,1}); % size of coef vector(matrix) in cell-array
    num_coeff = s1*s2*t1*t2;
    r = reshape(y_thresh(1:num_coeff),t1*s1,s2*t2);
    coef_cell{zz,1} = mat2cell(r,ones(1,s1)*t1, ones(1,s2)*t2);
    y_thresh(1:num_coeff) = [];
end

coef_thresh = coef_cell;