function [b,num_nnz] = thresh_matrix(a,J,N,request)

% global thresholding in matrix structure
%
% INPUT
% a - matrix in usual multiscale structure (up left corner is lowpass)
% J - level
% N - coefficients after shrinkage
% request - optional string 'thresh', then N is the treshold parameter
%                                      (not the number of coefficient!)
%
% OUTPUT
% b - thresholded matrix
% num_nnz - number of nnz coefficients in b
%
%
% 2009-03-16 (jens.krommweh@uni-due.de)

a1 = a;
[n1,n2] = size(a);

% ### I) set lowpass part = 0 in order to prohibit shrinkage there ###

lowpass = a(1:ceil(n1/2^J),1:ceil(n2/2^J));
a(1:ceil(n1/2^J),1:ceil(n2/2^J)) = 0;

% ### II) global hard thresholding ###

if nargin == 4
    % case 1:  threshold is given
    b = (abs(a) > N) .* a;
else % case 2: desired number of coefficients is given
    x = abs(a1(:));
    %y = sort(x,'descend');
    y = -sort(-x); % for old matlab versions
    m = y(floor(N));
    b = (abs(a) > m) .* a;
end

% ### III) add lowpass part ###

b(1:ceil(n1/2^J),1:ceil(n2/2^J)) = lowpass;

num_nnz = nnz(b); % non zero coefficients